package org.jtheque.films.stats.view.impl.panels;

/*
 * This file is part of JTheque.
 *
 * JTheque is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License.
 *
 * JTheque is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with JTheque.  If not, see <http://www.gnu.org/licenses/>.
 */

import org.jdesktop.swingx.JXHeader;
import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartPanel;
import org.jfree.chart.JFreeChart;
import org.jfree.ui.RectangleAnchor;
import org.jtheque.core.managers.Managers;
import org.jtheque.core.managers.resource.IResourceManager;
import org.jtheque.core.managers.resource.ImageType;
import org.jtheque.core.managers.language.Internationalizable;
import org.jtheque.core.managers.language.ILanguageManager;
import org.jtheque.core.utils.ui.PanelBuilder;
import org.jtheque.films.stats.StatsModule;
import org.jtheque.films.stats.services.able.IStatsService;
import org.jtheque.utils.ui.GridBagUtils;

import javax.annotation.Resource;
import javax.annotation.PostConstruct;
import javax.swing.JPanel;
import java.awt.Color;
import java.awt.Component;

/**
 * A stats panel for realizers.
 *
 * @author Baptiste Wicht
 */
public final class JPanelStatsRealizers extends JPanel implements PanelStats, Internationalizable {
    private static final long serialVersionUID = 2330748896262486492L;

    @Resource
    private IStatsService statsService;

    /* Instances */
    private final ILanguageManager resources = Managers.getManager(ILanguageManager.class);

    private JXHeader header;

    private JFreeChart pieRealizersByCountry;
    private JFreeChart pieRealizersByNote;

    /**
     * Build the panel.
     */
    @PostConstruct
    private void build() {
        Managers.getManager(ILanguageManager.class).addInternationalizable(this);
        
        PanelBuilder builder = new PanelBuilder(this);

        header = new JXHeader();
        header.setTitle(resources.getMessage("stats.realizers.title"));
        header.setDescription(statsService.getTitles()[0]);
        header.setIcon(Managers.getManager(IResourceManager.class).getIcon(
                StatsModule.IMAGE_BASENAME, "stats-big", ImageType.JPG));

        builder.add(header, builder.gbcSet(0, 0));

        pieRealizersByCountry = ChartFactory.createPieChart3D(
                resources.getMessage("stats.realizers.pie.country"),
                statsService.getInformations().getStatsOfRealizersByCountry(), true, true, true);

        pieRealizersByCountry.getLegend().setLegendItemGraphicLocation(RectangleAnchor.RIGHT);

        Component panelPieRealizersByCountry = new ChartPanel(pieRealizersByCountry);
        panelPieRealizersByCountry.setBackground(Color.white);

        builder.add(panelPieRealizersByCountry, builder.gbcSet(0, 1, GridBagUtils.BOTH, GridBagUtils.BASELINE, 1.0, 0.5));

        pieRealizersByNote = ChartFactory.createPieChart3D(resources.getMessage("stats.realizers.pie.note"),
                statsService.getInformations().getStatsOfRealizersByNote(), true, true, true);

        pieRealizersByNote.getLegend().setLegendItemGraphicLocation(RectangleAnchor.RIGHT);

        Component panelPieRealizersByNote = new ChartPanel(pieRealizersByNote);
        panelPieRealizersByNote.setBackground(Color.white);

        builder.add(panelPieRealizersByNote, builder.gbcSet(0, 2, GridBagUtils.BOTH, GridBagUtils.BASELINE, 1.0, 0.5));
    }

    @Override
    public void refreshText() {
        header.setTitle(resources.getMessage("stats.realizers.title"));
        header.setDescription(statsService.getInformations().getNumberOfRealizers() + " " +
                resources.getMessage("stats.realizers.realizers").toLowerCase(
                        Managers.getManager(ILanguageManager.class).getCurrentLocale()));

        pieRealizersByCountry.setTitle(resources.getMessage("stats.realizers.pie.country"));
        pieRealizersByNote.setTitle(resources.getMessage("stats.realizers.pie.note"));
    }

    /**
     * Return the header of the panel.
     *
     * @return The header of the panel.
     */
    public JXHeader getHeader() {
        return header;
    }
}